!w =================================================================================================================================
! MODULE       : erosion
!
! CONTACT      : Ronny Lauerwald, Haicheng Zhang
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       This module calculating the daily water-erosion rate.
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) :
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================
!!=========================================================================================
!!! Latest changes ------- by ZHC
!!! Time = 20170809
!!! Change contents: 
!!!   1) The eroded soil carbon and belowground litter is calculated based on the carbon and litter contents in the top 7 soil layers,
!!!   rather than based on the carbon and litter contents in the top 8 soil layers in previous version.
!!=========================================================================================
MODULE erosion
  !
  !
  ! routines called : restput, restget
  !
  USE ioipsl   
  USE xios_orchidee
  USE ioipsl_para 
  USE constantes
  USE constantes_soil
  USE constantes_var
  USE pft_parameters
  USE sechiba_io
  USE slowproc
  USE interpol_help
  USE grid
  USE mod_orchidee_para

  IMPLICIT NONE
 
  ! public routines :
 
  PRIVATE
  PUBLIC :: erosion_initialize, erosion_main, erosion_clear
 
  ! variables shared by all subroutines inside the erosion module : declaration and initialisation 
  LOGICAL, SAVE                                   :: l_first_erosion=.TRUE.      !! Logical to initialize the routine.
                                                                                 !! Initialisation has to be done one 
										    								     !! time (true/false)
  REAL(r_std), PARAMETER                          :: std_runoff = 10.0           !! daily total runoff (mm)
  REAL(r_std), PARAMETER                          :: std_qp = 0.001              !! daily peak runoff (m/s)
  REAL(r_std), PARAMETER                          :: std_cf = 0.10               !! standard vegetation cover factor 
  REAL(r_std), PARAMETER                          :: maxsoildep = 2.0            !! maximum soil depth (m)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)  :: gridarea                    !! grid area (m^2)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)  :: cveg_f                      !! vegetation cover factor, unitless, 0-1
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)  :: kflitter                    !! vegetation cover factor, unitless, 0-1
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)  :: kfroot                    !! vegetation cover factor, unitless, 0-1
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)    :: meanrunoff                  !! Accumulated runoff(kg/m^{-2} = mm) 
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex		
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)    :: peakrunoff                  !! Peak runoff over one day (mm/s) 
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)    :: meanrundra                  !! Accumulated runoff+drainage(kg/m^{-2} = mm) 
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex		
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)    :: peakrundra                  !! Peak (runoff +drainage) over one day (mm/s) 
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex															 
  INTEGER(r_std), SAVE, ALLOCATABLE, DIMENSION(:) 		:: no_events             !! number of rainfall events
                                                                                 !! @tex $(kg m^{-2} dt^{-1})$ @endtex	
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)        :: eroc_active         	 !! Active SOC export (gC/m^2/day)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)        :: eroc_slow         	 !! Slow SOC export (gC/m^2/day)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)        :: eroc_passive          !! Passive SOC export (gC/m^2/day)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:,:)      :: tot_7c                !! Total Carbon concentration in the top 7 soil layers  (gC m^{-2})
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:,:)      :: tot_7litbelow         !! Total belowground litter in the top 7 soil layers (gC m^{-2})
  !!********************* End veraibles for history files ***********************
CONTAINS
!! 1. Initialize
  SUBROUTINE erosion_initialize(kjit,nbpt,index,rest_id, lalo,neighbours,resolution,contfrac,std_totsed,gridarea,effgrid_perc)
	!!
	IMPLICIT NONE
!	LOGICAL, SAVE                                   :: l_first_erosion=.TRUE.      !! Logical to initialize the routine.
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)     :: kjit                 !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)     :: nbpt                 !! Domain size (unitless)
    INTEGER(i_std), INTENT(in)     :: index(nbpt)          !! Indices of the points on the map (unitless)
    INTEGER(i_std),INTENT(in)      :: rest_id              !! Restart file identifier (unitless)
	REAL(r_std), INTENT(in)        :: lalo(nbpt,2)              !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), INTENT(in)     :: neighbours(nbpt,8)        !! Vector of neighbours for each grid point 
                                                                !! (unitless; 1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW)
    REAL(r_std), INTENT(in)        :: resolution(nbpt,2)        !! The size of each grid box in X and Y (m)
	REAL(r_std), INTENT(in)        :: contfrac(nbpt)            !! Fraction of land in each grid box (unitless;0-1)
	!! Output variables
	REAL(r_std), INTENT(out)       :: std_totsed(nbpt)        !! standard sediment discharge amount rate for each ORCHIDEE cell(ton)
	REAL(r_std), INTENT(out)       :: gridarea(nbpt)          !! Area of each grid-cell ()
	REAL(r_std), INTENT(out)       :: effgrid_perc(nbpt)      !! percentage of the effective area in a ORCHIDEE pixel (0-1)
    !! 1.1 First call only
	WRITE(numout,*) 'Start the first time calling of erosion module', l_first_erosion	
    IF (l_first_erosion) THEN
		WRITE(numout,*) 'This is the first time calling of erosion module'	
		ALLOCATE(cveg_f(nbpt,nvm))
		cveg_f = zero
		ALLOCATE(kflitter(nbpt,nvm))
		kflitter = zero
		ALLOCATE(kfroot(nbpt,nvm))
		kfroot = zero
		ALLOCATE(peakrunoff(nbpt))
		peakrunoff = zero		
		ALLOCATE(meanrunoff(nbpt))
		meanrunoff = zero
		ALLOCATE(peakrundra(nbpt))
		peakrundra = zero		
		ALLOCATE(meanrundra(nbpt))
		meanrundra = zero
		ALLOCATE(eroc_active(nbpt,nvm))
		eroc_active = zero
		ALLOCATE(eroc_slow(nbpt,nvm))
		eroc_slow = zero
		ALLOCATE(eroc_passive(nbpt,nvm))
		eroc_passive = zero
		ALLOCATE(tot_7c(nbpt,ncarb,nvm))
		tot_7c = zero
		ALLOCATE(tot_7litbelow(nbpt,nlitt,nvm))
		tot_7litbelow = zero	
		ALLOCATE(no_events(nbpt))		
		no_events = 0
		! Reading input data for calculating erosion (flacc, fldir, std_totsed)
		WRITE(numout,*) 'Start read erosion map'	
		CALL erosionmap(nbpt,index,lalo,neighbours,resolution,contfrac,std_totsed,gridarea,effgrid_perc)
		! Test reading input data
!        WRITE(numout,*) 'std_totsed', std_totsed(:)
!		WRITE(numout,*) 'gridarea', gridarea(:)
!		WRITE(numout,*) 'effgrid_perc', effgrid_perc(:)
		!
		l_first_erosion = .FALSE.
		WRITE(numout,*) 'End first_erosion'
	ENDIF !l_first_erosion
  END SUBROUTINE erosion_initialize
!!																							
  SUBROUTINE erosion_main(kjit,nbpt,rest_id,dt_sechiba,index, ldrestart_read, ldrestart_write, &
       & lalo,neighbours,resolution,temp_sol,bulkdens,laydep,contfrac,runoff,drainage,veget,veget_max,hist_id, &
	   & std_totsed,gridarea,effgrid_perc,rootmass,litter_above,litter_below,carbon,cf_veget,kf_litter,kf_root, &
	   & totrunoff_d,peakrunoff_d,totrundra_d,peakrundra_d,erocap,erodep,eroc,tot_erocap,ave_erodep,tot_eroc)
    IMPLICIT NONE
	!WRITE(numout,*) 'Erosion simulation0', fldir(1238726)
    !! 0.1 Input variables
	INTEGER(i_std), INTENT(in)     :: kjit                      !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)     :: nbpt                      !! Domain size  (unitless)
    INTEGER(i_std),INTENT(in)      :: rest_id                   !! Restart file identifier (unitless)
	REAL(r_std), INTENT(in)        :: runoff(nbpt)         		!! Grid-point runoff @tex $(kg m^{-2} dt^{-1}, mm/dt)$ @endtex	
	REAL(r_std), INTENT(in)        :: drainage(nbpt)         	!! Grid-point drainage @tex $(kg m^{-2} dt^{-1}, mm/dt)$ @endtex	
    REAL(r_std), INTENT(in)        :: dt_sechiba        		!! Time step of the model (s)	
    INTEGER(i_std), INTENT(in)     :: index(nbpt)               !! Indices of the points on the map (unitless)
	REAL(r_std), INTENT(in)        :: lalo(nbpt,2)              !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), INTENT(in)     :: neighbours(nbpt,8)        !! Vector of neighbours for each grid point 
                                                                !! (unitless; 1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW)
    REAL(r_std), INTENT(in)        :: resolution(nbpt,2)        !! The size of each grid box in X and Y (m)
	REAL(r_std), INTENT(in)        :: contfrac(nbpt)            !! Fraction of land in each grid box (unitless;0-1)
	INTEGER(i_std),INTENT(in)      :: hist_id                   !! Access to history file (unitless)
	REAL(r_std),INTENT (in)        :: veget(nbpt,nvm)           !! Fraction of vegetation type (unitless;0-1)
    REAL(r_std), INTENT(in)        :: veget_max(nbpt,nvm)       !! Maximal fraction of vegetation (unitless;0-1)	
    LOGICAL, INTENT(in)            :: ldrestart_read       		!! Logical for _restart_ file to read (true/false)
    LOGICAL, INTENT(in)            :: ldrestart_write      		!! Logical for _restart_ file to write (true/false)
    REAL(r_std), INTENT(in)        :: temp_sol(nbpt)            !! Surface temperature (K)	
	REAL(r_std), INTENT (inout)    :: bulkdens(nbpt)            !! bulk_density (kg m-3)    zhc
	REAL(r_std),INTENT (inout)     :: laydep(nbdl)              !! The lower limit of each (11) soil layer (m) ::zhc
	!!
    !!0.2 modified variables
	REAL(r_std),DIMENSION (nbpt,nvm,nparts,nelements), INTENT (inout)      :: rootmass       	!! belowground biomass
	                                                                                            !! @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
	REAL(r_std),DIMENSION (nbpt,nlitt,nvm,nelements), INTENT (inout)      :: litter_above       !! Above ground metabolic and structural litter 
	                                                                                            !! @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
    REAL(r_std),DIMENSION (nbpt,nlitt,nvm,nbdl,nelements), INTENT (inout) :: litter_below       !! Below ground metabolic and structural litter 
                                                                                                !! per ground area @tex $(gC m^{-2})$ !!ORCHIDEE grid
	REAL(r_std),DIMENSION (nbpt,ncarb,nvm,nbdl), INTENT (inout)           :: carbon             !! Soil carbon pools per ground area: active, slow, or 
                                                                                                !! passive, @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
	REAL(r_std), INTENT(in)       :: std_totsed(nbpt)        !! standard sediment discharge amount rate for each ORCHIDEE cell(ton)
	REAL(r_std), INTENT(in)       :: effgrid_perc(nbpt)      !! percentage of the effective area in a ORCHIDEE pixel (0-1)
	REAL(r_std), INTENT(in)       :: gridarea(nbpt)          !! area of each grid cell (m^2)
	!!
	!! 0.3 output variables
	REAL(r_std), INTENT(out)   :: totrunoff_d(nbpt)                               !! daily total runoff (mm)
	REAL(r_std), INTENT(out)   :: peakrunoff_d(nbpt)                              !! daily peak runoff rate.(mm/s = kg/m-2/s)
	REAL(r_std), INTENT(out)   :: totrundra_d(nbpt)                               !! daily total runoff+drainage (mm)
	REAL(r_std), INTENT(out)   :: peakrundra_d(nbpt)                              !! daily peak (runoff+drainage) rate.(mm/s = kg/m-2/s)
	REAL(r_std), INTENT(out)   :: erocap(nbpt,nvm)                            	  !! Erosion capacity from subgrid.(ton)
	REAL(r_std), INTENT(out)   :: erodep(nbpt,nvm)                                !! Erosion depth of sub_grid (m)
	REAL(r_std), INTENT(out)   :: eroc(nbpt,ncarb,nvm)                            !! Carbon export (gC/m^2/day)
	REAL(r_std), INTENT(out)   :: tot_erocap(nbpt)                            	  !! Erosion capacity from subgrid.(ton)
	REAL(r_std), INTENT(out)   :: ave_erodep(nbpt)                                !! Erosion depth of sub_grid (m)
	REAL(r_std), INTENT(out)   :: tot_eroc(nbpt,ncarb)                             	  !! Carbon export (kg/day)
	REAL(r_std), INTENT(out)   :: cf_veget(nbpt,nvm)                              !! Daily vegetation cover factor
	REAL(r_std), INTENT(out)   :: kf_litter(nbpt,nvm)                             !! Daily aboveground litter cover factor
	REAL(r_std), INTENT(out)   :: kf_root(nbpt,nvm)                             !! Daily belowground litter cover factor
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	!! 0.4 Local variables					
    INTEGER(i_std)             :: ig,jg,lg,fg,ng,ii,icarb,ilitt,ibdl !! integers for loops
	INTEGER(i_std)             :: ilat,ilon,inbpt		             !! temperal variable for latitude and longitude
	INTEGER(i_std)             :: max_flacc		             	  !! maximum value of flow accumulation
	REAL(r_std)                :: erocap_jg                       !! temperal variable for erosion capacity
	REAL(r_std)                :: meanrun_up,peakrun_up,meanrundra_up,peakrundra_up,cveg_up,kflitter_up,kfroot_up !! temperal variable for erosion capacity
	REAL(r_std)                :: littotal,belowbiom               !! vegetation cover fraction (0-1)
	REAL(r_std)                :: vegf                 			  !! vegetation cover fraction (0-1)
!	REAL(r_std)                :: ave_c7,ave_lit                  !! temperal variable for erosion capacity
!	REAL(r_std)                :: nsubcell			        	  !! potentially maximum number of finer cells(erosion map)
    REAL(r_std), SAVE          :: time_counter                    !! Time counter (s)
    CHARACTER(LEN=80)          :: var_name      	              !! To store variables names for I/O (unitless)	
	CHARACTER(LEN=10)          :: carb_suff                       !! To assign a suffix to the variables name indicating carbon type
	                                                              !! (active, slow, passive)
	CHARACTER(LEN=10)          :: litt_suff                       !! To assign a suffix to the variables name indicating litterfall type
	                                                              !! (structual, metabolic)
    CHARACTER(LEN=10)          :: solly_suff                      !! To store variables names for I/O soil layer																
	REAL(r_std), DIMENSION(nbpt,ncarb,nvm,nbdl)  ::carbon_old         !!Soil carbon pools per ground area: active, slow, or 
                                                                                                !! passive, @tex $(gC m^{-2})$ @endtex !!ORCHIDEE grid
    !
	laydep(nbdl) = maxsoildep
	DO ig = 1,nbdl-1
		laydep(ig) = maxsoildep/(2**(nbdl-1)-1)*((2**(ig-1) -1)+(2**(ig)-1))/2.0
	ENDDO		
	!		WRITE(numout,*) 'Depth_Soillayer', laydep(:)
	
!! 1.1 Aggregate runoff values per day. Extract the daily total runoff amount and peak runoff rate
!! peak runoff rate = maximum of the 48 half-hour runoff rates winthin a day	
	DO ig=1,nbpt
		! Only runoff considered
		meanrunoff(ig) = meanrunoff(ig) + runoff(ig)
		IF (runoff(ig) .GT. peakrunoff(ig)) THEN
			peakrunoff(ig) = runoff(ig)
		ENDIF !(runoff(ig) .GT. peakrunoff(ig))
		
		IF (runoff(ig) .GT. min_sechiba) THEN
			no_events(ig) = no_events(ig) + 1
		ENDIF !(runoff(ig) .GT. min_sechiba)
		!
		! Both runoff and drainage considered
		meanrundra(ig) = meanrundra(ig) + runoff(ig)+drainage(ig)
		IF (runoff(ig)+drainage(ig) .GT. peakrundra(ig)) THEN
			peakrundra(ig) = runoff(ig)+drainage(ig)
		ENDIF !(runoff(ig) .GT. peakrunoff(ig))
	ENDDO !ig=1,nbpt
	!
    time_counter = time_counter + dt_sechiba 
!	WRITE(numout,*) 'time_counter',time_counter,'dt_sechiba',dt_sechiba
!	WRITE(numout,*) 'runoff',runoff(1),'meanrunoff',meanrunoff(1),'peakrunoff',peakrunoff(1)
    !
!! 2 If the simulation for the whole 48 half-hour intervals of a day have been finished,
!!   we calculate the erosion.
   !! 2.1 output daily mean runoff and the peak runoff rate
    !
    IF ( NINT(time_counter) .GE. NINT(one_day) ) THEN 
		peakrunoff(:) = peakrunoff(:) / dt_sechiba
		peakrundra(:) = peakrundra(:) / dt_sechiba
	!!	peakrunoff(:) = peakrunoff(:)
		WHERE (peakrunoff(:) .LT. min_sechiba)
		peakrunoff(:) = zero
		ENDWHERE
		WHERE (peakrundra(:) .LT. min_sechiba)
		peakrundra(:) = zero
		ENDWHERE
	!	
		WHERE (meanrunoff(:) .LT. min_sechiba)
		meanrunoff(:) = zero
		ENDWHERE
		WHERE (meanrundra(:) .LT. min_sechiba)
		meanrundra(:) = zero
		ENDWHERE
		totrunoff_d(:)=meanrunoff(:)
		peakrunoff_d(:)=peakrunoff(:)
		totrundra_d(:)=meanrundra(:)
		peakrundra_d(:)=peakrundra(:)
	!	
	!	WRITE(numout,*) 'no of events: ', no_events(:)			
	!	WRITE(numout,*) 'peak runoff was: ', peakrunoff(:)	
	!	WRITE(numout,*) 'mean runoff was: ', meanrunoff(:)
	!	WRITE(numout,*) 'peak runoff was: ', peakrundra(:)	
	!	WRITE(numout,*) 'mean runoff was: ', meanrundra(:)
	!	WRITE(numout,*) 'bulk_density3 was: ', bulkdens(:)
	!   WRITE(numout,*) 'litter_above: ', litter_above(:,:,:,:)
	!   WRITE(numout,*) 'Biomass: ', rootmass(:,:,:,1)
	!	WRITE(numout,*) 'layer_depth1: ', laydep(7)
	!
    ! 2.2 Calculate vegetation, litter, root factor
	! 2.2.1 Calculate vegetation and management factor
		cf_veget(:,:)=zero
		DO jg=1,nbpt
			cveg_f(jg,1)=0.0
			DO lg=2,nvm
				vegf=100.0*veget(jg,lg)/veget_max(jg,lg)
				IF (vegf.LT. 0.11) THEN
					cveg_f(jg,lg)=1.0
				ELSEIF (vegf .GT. 95.) THEN
					cveg_f(jg,lg)=0.01
				ELSE
					cveg_f(jg,lg)=0.650-0.30*log10(vegf)
				ENDIF !(veget(jg,2:nvm) .LT. 0.11)
			ENDDO !DO lg=2,nvm
		ENDDO !jg=1,nbpt
		cf_veget(:,:)=cveg_f(:,:)
	! 2.2.2 Calculate factors of aboveground litter and belowground litter
		DO jg=1,nbpt
		    kflitter(jg,1)=1.0
		    kfroot(jg,1)=1.0
		    DO lg=2,nvm
				IF (lg.LT.12) THEN
					littotal=0.001*(sum(litter_above(jg,:,lg,icarbon))+sum(litter_below(jg,:,lg,1:7,icarbon)))
				ELSE
					littotal=0.001*sum(litter_below(jg,:,lg,1:7,icarbon))
				ENDIF
				!
				IF (veget_max(jg,lg).GT.zero) THEN
					kflitter(jg,lg)=EXP(-0.56*littotal)
				ELSE
					kflitter(jg,lg)=0.0
				ENDIF
				!
				belowbiom=0.001*(rootmass(jg,lg,isapbelow,icarbon)+ &
					&	rootmass(jg,lg,iheartbelow,icarbon)+rootmass(jg,lg,iroot,icarbon))
		        IF (veget_max(jg,lg).GT.zero) THEN
					kfroot(jg,lg)=EXP(-0.56*belowbiom)
				ELSE
					kfroot(jg,lg)=0.0
				ENDIF
		    ENDDO !lg=2,nvm
		ENDDO !jg=1,nbpt
		kf_litter(:,:)=kflitter(:,:)
		kf_root(:,:)=kfroot(:,:)
    !
    !! 2.3 Calculate erosion rate based on MUSLE equation
	!    2.3.1  Calculate the sediment amount(ton) for each finer(sub-) grid (standard erosion grid)
		erocap(:,:)=zero
		tot_erocap(:)=zero
		erodep(:,:)=zero
		ave_erodep(:)=zero
		DO ig=1,nbpt
			meanrun_up=totrunoff_d(ig)!
			peakrun_up=peakrunoff_d(ig)!
			IF (temp_sol(ig).LT.(-5.0)) THEN
				erocap(ig,:)=zero
				erodep(ig,:)=zero
				tot_erocap(ig)=zero
				ave_erodep(ig)=zero
			ELSE
				DO lg=1,nvm
					IF (veget_max(ig,lg).GT.zero) THEN
						! Only surface runoff considered
						cveg_up=cf_veget(ig,lg)
						kflitter_up=kf_litter(ig,lg)
						kfroot_up=kf_root(ig,lg)
						erocap(ig,lg)=(meanrun_up*peakrun_up/(std_runoff*std_qp)/1000.)**0.56 &
						&        *(cveg_up/std_cf)*std_totsed(ig)*kflitter_up*kfroot_up*veget_max(ig,lg)    !*kfroot_up   % unit ton/day (Mg/day)
						! surface runoff+drainage
						!	erocap(ig)=(meanrundra_up*peakrundra_up/(std_runoff*std_qp)/1000)**0.56 &
						!     &        *(cveg_up/std_cf)*std_totsed(ig)*kflitter_up    !*kfroot_up
						IF (erocap(ig,lg).LT.zero) THEN
							erocap(ig,lg)=zero
						ENDIF
						! depth of the sediment if all input sediments from upstream are deposited at one grid-cell
						tot_erocap(ig)=SUM(erocap(ig,:)) ! ton/day
						IF (effgrid_perc(ig).GT.zero) THEN
							erodep(ig,lg)=1000.*erocap(ig,lg)/(gridarea(ig)*veget_max(ig,lg)*effgrid_perc(ig))/bulkdens(ig) !m/day
							ave_erodep(ig)=1000.*tot_erocap(ig)/(gridarea(ig)*effgrid_perc(ig))/bulkdens(ig) ! m/day
						ENDIF
					!
					ENDIF !(veget_max(ig,lg).GT.zero) THEN
				ENDDO !DO lg=1,nvm
			ENDIF !IF (temp_sol(ig).LT.(-2.0)) THEN
		ENDDO !DO lg=1,nbpt
	!	WRITE(numout,*) 'erosion depth', ave_erodep(:)
		!
		!2.4 Calculate the processes of erosion rate of SOC
		!      Calculate from the upstream to outlets of the watershed based on flow accumulation
		!WRITE(numout,*) 'erosion_carbon', carbon(:,1,:,1)
		carbon_old(:,:,:,:)=zero
		tot_eroc(:,:)=zero
		carbon_old(:,:,:,:)=carbon(:,:,:,:)
		eroc(:,:,:)=zero
	    DO ig=1,nbpt
			DO lg=1,nvm
				IF (ISNAN(SUM(carbon(ig,3,lg,:)))) THEN
					carbon(ig,:,lg,:)=zero
					carbon_old(ig,:,lg,:)=zero
				ENDIF !
				!
				IF ((veget_max(ig,lg) .GT. 0).AND.(erodep(ig,lg).GT.0).AND.(MinVal(carbon(ig,:,lg,1:7)).GT.0)) THEN  !!! 1<=fldir<=128
					tot_7c(ig,:,lg)=carbon(ig,:,lg,1)+carbon(ig,:,lg,2)+carbon(ig,:,lg,3)+ &
					&   	carbon(ig,:,lg,4)+carbon(ig,:,lg,5)+carbon(ig,:,lg,6)+carbon(ig,:,lg,7)
!					tot_7litbelow(ig,:,lg)=litter_below(ig,:,lg,1,icarbon)+litter_below(ig,:,lg,2,icarbon)+ &
!					&   	litter_below(ig,:,lg,3,icarbon)+litter_below(ig,:,lg,4,icarbon)+litter_below(ig,:,lg,5,icarbon)+ &
!					&   	litter_below(ig,:,lg,6,icarbon) + litter_below(ig,:,lg,7,icarbon)
					eroc(ig,:,lg)=erodep(ig,lg)/laydep(7)*tot_7c(ig,:,lg)
					tot_eroc(ig,:)=tot_eroc(ig,:)+ eroc(ig,:,lg)*veget_max(ig,lg)
					DO jg=1,7 ! top 7 soil layers
						carbon(ig,:,lg,jg)=carbon_old(ig,:,lg,jg)-eroc(ig,:,lg)*carbon_old(ig,:,lg,jg)/tot_7c(ig,:,lg)+ &
						&		(erodep(ig,lg)/(laydep(8)-laydep(7))*carbon_old(ig,:,lg,8))*carbon_old(ig,:,lg,jg)/tot_7c(ig,:,lg)
					ENDDO ! top 7 soil layers
					DO jg=10,8,-1 ! top 8-10 soil layers
						carbon(ig,:,lg,jg)=carbon_old(ig,:,lg,jg)-erodep(ig,lg)/(laydep(jg)-laydep(jg-1))*carbon_old(ig,:,lg,jg)+ &
									erodep(ig,lg)/(laydep(jg+1)-laydep(jg))*carbon_old(ig,:,lg,jg+1)
					ENDDO
					carbon(ig,:,lg,11)=carbon_old(ig,:,lg,11)-erodep(ig,lg)/(laydep(11)-laydep(10))*carbon_old(ig,:,lg,11)
				!	WRITE(numout,*) 'ero_totc', ig,lg,erodep(ig,lg),laydep(7), SUM(tot_7c(ig,:,lg)),SUM(eroc(ig,:,lg))
				!	WRITE(numout,*) 'soilcarbon_passive', carbon(ig,3,lg,1:7)
				!	WRITE(numout,*) 'soilcarbon_active', carbon(ig,1,lg,1:7)	
				ENDIF
                        !        WRITE(numout,*) 'ero_totc', veget_max(ig,lg),MinVal(carbon(ig,:,lg,1:7)),SUM(tot_eroc(ig,:))
			ENDDO !DO lg=1,nvm
		!	WRITE(numout,*) 'erosion carbon', SUM(tot_eroc(ig,:))
	    ENDDO !DO ig=1,nbpt
		eroc_active(:,:)=eroc(:,1,:)
		eroc_slow(:,:)=eroc(:,2,:)
		eroc_passive(:,:)=eroc(:,3,:)
    !
    !! 2.6 Write soil & carbon erosion rate to the history file
	!
	    CALL xios_orchidee_send_field("bulkdens",bulkdens)
        CALL xios_orchidee_send_field("totrunoff_d",totrunoff_d)
	    CALL xios_orchidee_send_field("peakrunoff_d",peakrunoff_d)
		CALL xios_orchidee_send_field("totrundra_d",totrundra_d)
	    CALL xios_orchidee_send_field("peakrundra_d",peakrundra_d)
		CALL xios_orchidee_send_field("cf_veget",cf_veget)
		CALL xios_orchidee_send_field("kf_litter",kf_litter)
		CALL xios_orchidee_send_field("kf_root",kf_root)
		CALL xios_orchidee_send_field("erocap",erocap)
		CALL xios_orchidee_send_field("erodep",erodep)
		CALL xios_orchidee_send_field("eroc_active",eroc_active)
		CALL xios_orchidee_send_field("eroc_slow",eroc_slow)
		CALL xios_orchidee_send_field("eroc_passive",eroc_passive)
		CALL xios_orchidee_send_field("tot_erocap",tot_erocap)
		CALL xios_orchidee_send_field("ave_erodep",ave_erodep)
		CALL xios_orchidee_send_field("tot_eroc",tot_eroc)
	!	
		! Reinitialize values
		time_counter = zero
		cveg_f = zero
		cf_veget=zero
		kflitter=zero
		kfroot=zero
		kf_litter = zero
		kf_root = zero
		peakrunoff = zero
		meanrunoff = zero
		peakrundra = zero
		meanrundra = zero
		erocap = zero
		erodep = zero
		eroc=zero
		tot_erocap=zero
		ave_erodep =zero
		tot_eroc =zero
		no_events = 0
		
		WRITE(numout,*) 'erosion END'

!
!		WRITE(numout,*) 'End of this day'
	ENDIF !	NINT(time_counter) .GE. NINT(dt_routing)
    		
    !  
  END SUBROUTINE erosion_main		
    !
    !
!! ================================================================================================================================
!! SUBROUTINE   : erosion_clear
!!
!> BRIEF        : This subroutine deallocates the block memory previously allocated.
!! \n
!_ ================================================================================================================================ 
  SUBROUTINE erosion_clear()
    l_first_erosion = .TRUE.
	!
	IF (ALLOCATED(gridarea))           DEALLOCATE(gridarea)
    IF (ALLOCATED(cveg_f))             DEALLOCATE(cveg_f)
	IF (ALLOCATED(peakrunoff))         DEALLOCATE(peakrunoff)
    IF (ALLOCATED(meanrunoff))         DEALLOCATE(meanrunoff)
	IF (ALLOCATED(peakrundra))         DEALLOCATE(peakrundra)
    IF (ALLOCATED(meanrundra))         DEALLOCATE(meanrundra)	
    IF (ALLOCATED(no_events))          DEALLOCATE(no_events)
	IF (ALLOCATED(eroc_active))        DEALLOCATE(eroc_active)
	IF (ALLOCATED(eroc_slow))          DEALLOCATE(eroc_slow)
	IF (ALLOCATED(eroc_passive))       DEALLOCATE(eroc_passive)
	IF (ALLOCATED(tot_7c))             DEALLOCATE(tot_7c)
	IF (ALLOCATED(tot_7litbelow))      DEALLOCATE(tot_7litbelow)
	!**************** output variable *************************
  END SUBROUTINE erosion_clear
	!
	!
	!
!! ================================================================================================================================
!! SUBROUTINE   : erosionmap
!!
!> BRIEF        : This subroutine reads and interpolates erosion map
!! \n
!_ ================================================================================================================================ 
  SUBROUTINE erosionmap(nbpt,index,lalo,neighbours,resolution,contfrac,std_totsed,gridarea,effgrid_perc)
    !
    IMPLICIT NONE
	!
	INTEGER(i_std), INTENT(in)                     :: nbpt                  !! Domain size  (unitless)  
    INTEGER(i_std), INTENT(in)                     :: index(nbpt)           !! Index on the global map.
	REAL(r_std), INTENT(in)                        :: lalo(nbpt,2)          !! Vector of latitude and longitudes (beware of the order !)
	INTEGER(i_std), INTENT(in)                     :: neighbours(nbpt,8)    !! Vector of neighbours for each grid point (1=N, 2=E, 3=S, 4=W)
!    INTEGER(i_std), INTENT(out)                    :: maxnbp                !! maxnbp(1):number of fine grid in a ORCHIDEE grid, 
	                                                                        !! 2,3: total row and column number of sub_grid
	REAL(r_std), INTENT(in)                        :: resolution(nbpt,2)    !! The size of each grid box in X and Y (m)	
    REAL(r_std), INTENT(in)                        :: contfrac(nbpt)        !! Fraction of land in each grid box (unitless;0-1)
	CHARACTER(LEN=80)                              :: filename               !! Name of the netcdf file (unitless)
	INTEGER(i_std)                                 :: nbpmax, nix, njx, fopt !! IDices, (unitless)
	CHARACTER(LEN=30)                              :: callsign               !!
	REAL(r_std), INTENT(out)                       :: std_totsed(nbpt)      !! standard sediment discharge amount rate for each ORCHIDEE cell(ton)
	REAL(r_std), INTENT(out)                       :: effgrid_perc(nbpt)    !! percentage of the effective area in a ORCHIDEE pixel (0-1)
	REAL(r_std), INTENT(out)                       :: gridarea(nbpt)        !! area of each grid cell (m^2)
	!
    INTEGER                                        :: ALLOC_ERR             !!
    LOGICAL                                        :: ok_interpol = .FALSE. !! Flag for interpolation (true/false)
    LOGICAL                                        :: is_root_prc_old		 !!
	INTEGER(i_std)                                 :: iml, jml, lml, tml, fid, ib,ig, jb, ip, jp, itype,subres !! Indices (unitless)
    REAL(r_std)                                    :: lev(1), date, dt, coslat !!
	REAL(r_std)                                    :: lon0,lat0,res_lon,res_lat,cellarea	    !! size (m) of target grid at longitude and latitude direction
	REAL(r_std)                                    :: sumsed				!! Total erosion from one ORCHIDEE cell in the simulation resolution (ton)
	REAL(r_std)                                    :: sumarea				!! total effective area (m^2)
    INTEGER(i_std)                                 :: itau(1)               !!
	
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: latrel                !! Latitude
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: lonrel                !! Longitude
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:)     :: resol_lu              !! Resolution read on the map
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: totsed          		!! erosion rate for the finer grid, read from forcing data(ton)
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: effgrid               !! percentage of the effective area in a ORCHIDEE pixel (0-1)
	REAL(r_std), ALLOCATABLE, DIMENSION(:,:)       :: ero_area              !! Area on the fine grid @tex $(m^2})$ @endtex
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:)  :: ero_index             !! Indices of the points we need on the fine grid (unitless)
	INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:)    :: mask                  !! Mask to exclude some points (unitless)
	!sub_gridarea, sub_index
!_ ================================================================================================================================	
	!
	is_root_prc_old=is_root_prc
	is_root_prc=.TRUE.
	!
    filename = 'erosion.nc' !
    CALL getin_p('EROSION_FILE',filename)  	
	!
	WRITE(numout,*) 'read info of erosion map', nbpt,is_root_prc, filename,iml, jml, lml, tml, fid
    IF (is_root_prc) THEN
       CALL flininfo(filename,iml, jml, lml, tml, fid)
       CALL flinclo(fid)
    ENDIF
    CALL bcast(iml)
    CALL bcast(jml)
    CALL bcast(lml)
    CALL bcast(tml)
	WRITE(numout,*) 'read info of erosion map', nbpt,is_root_prc, filename,iml, jml, lml, tml, fid
    !
	ALLOC_ERR=-1
    ALLOCATE (lonrel(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of lonrel: ",ALLOC_ERR, jml
      STOP 
    ENDIF
	lonrel(:,:)=zero
	!
    ALLOC_ERR=-1
    ALLOCATE (latrel(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of latrel: ",ALLOC_ERR, jml
      STOP 
    ENDIF
	latrel(:,:)=zero
	!
	ALLOC_ERR=-1
    ALLOCATE (totsed(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of totsed: ",ALLOC_ERR, jml
      STOP 
    ENDIF
	totsed(:,:)=zero
	!
	ALLOC_ERR=-1
    ALLOCATE (effgrid(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of effgrid: ",ALLOC_ERR, jml
      STOP 
    ENDIF
	effgrid(:,:)=zero
	!
	ALLOC_ERR=-1
    ALLOCATE(resol_lu(iml,jml,2), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of resol_lu : ",ALLOC_ERR
      STOP 
    ENDIF
	resol_lu(:,:,:)=zero
    !
    ALLOC_ERR=-1
    ALLOCATE(mask(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
      WRITE(numout,*) "ERROR IN ALLOCATION of mask : ",ALLOC_ERR
      STOP 
    ENDIF
	mask(:,:) = 0.
	!
    ALLOC_ERR=-1
    ALLOCATE(ero_index(nbpt, nbpmax, 2), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
       WRITE(numout,*) "ERROR IN ALLOCATION of ero_index : ",ALLOC_ERR
       STOP 
    ENDIF
    ero_index(:,:,:)=0
    !
	ALLOC_ERR=-1
    ALLOCATE(ero_area(nbpt, nbpmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR/=0) THEN
       WRITE(numout,*) "ERROR IN ALLOCATION of ero_area : ",ALLOC_ERR
       STOP 
    ENDIF
    ero_area(:,:)=zero
!	!
	WRITE(numout,*) "Open erosion file",is_root_prc, fid
    IF (is_root_prc) CALL flinopen(filename, .FALSE., iml, jml, lml, lonrel, latrel, lev, tml, itau, date, dt, fid)
	WRITE(numout,*) "Open erosion file1",iml, jml, lml,fid
    CALL bcast(lonrel)
    CALL bcast(latrel)
    CALL bcast(lev)
    CALL bcast(itau)
    CALL bcast(date)
    CALL bcast(dt)
    !
	WRITE(numout,*) "Open erosion totsed",iml, jml, lml, tml,is_root_prc,fid
    IF (is_root_prc) CALL flinget(fid, 'totsed', iml, jml, lml, tml, 1, 1, totsed)
	WRITE(numout,*) "Open erosion effgrid_perc",iml, jml, lml, tml,is_root_prc,fid
	IF (is_root_prc) CALL flinget(fid, 'effgrid_perc', iml, jml, lml, tml, 1, 1, effgrid)
	WRITE(numout,*) "Open erosion fid",iml, jml, lml, tml,is_root_prc,fid
    IF (is_root_prc) CALL flinclo(fid)
	!
	WRITE(numout,*) 'lat-forcingdata', lalo(1:4,1)
	WRITE(numout,*) 'lon-forcingdata', lalo(1:4,2)
	WRITE(numout,*) 'lonrel : ', MAXVAL(lonrel), MINVAL(lonrel),lonrel(1:2,1)
    WRITE(numout,*) 'latrel : ', MAXVAL(latrel), MINVAL(latrel),latrel(1,1:2)
!    WRITE(numout,*) 'erosion_rate : ', totsed(:,:)
!	WRITE(numout,*) 'effgrid : ', effgrid(:,:)
	!
	DO ip=1,iml
       DO jp=1,jml
          IF ( totsed(ip,jp).GT. min_sechiba ) THEN
             mask(ip,jp) = 1
          ENDIF
          ! Resolution in longitude
          !
          coslat = MAX( COS( latrel(ip,jp) * pi/180. ), mincos )     
          IF ( ip .EQ. 1 ) THEN
             resol_lu(ip,jp,1) = ABS( lonrel(ip+1,jp) - lonrel(ip,jp) ) * pi/180. * R_Earth * coslat
          ELSEIF ( ip .EQ. iml ) THEN
             resol_lu(ip,jp,1) = ABS( lonrel(ip,jp) - lonrel(ip-1,jp) ) * pi/180. * R_Earth * coslat
          ELSE
             resol_lu(ip,jp,1) = ABS( lonrel(ip+1,jp) - lonrel(ip-1,jp) )/2. * pi/180. * R_Earth * coslat
          ENDIF
          !
          ! Resolution in latitude
          !
          IF ( jp .EQ. 1 ) THEN
             resol_lu(ip,jp,2) = ABS( latrel(ip,jp) - latrel(ip,jp+1) ) * pi/180. * R_Earth
          ELSEIF ( jp .EQ. jml ) THEN
             resol_lu(ip,jp,2) = ABS( latrel(ip,jp-1) - latrel(ip,jp) ) * pi/180. * R_Earth
          ELSE
             resol_lu(ip,jp,2) =  ABS( latrel(ip,jp-1) - latrel(ip,jp+1) )/2. * pi/180. * R_Earth
          ENDIF
          !
       ENDDO
    ENDDO
	WRITE(numout,*) "lon_resolu=",MAXVAL(resol_lu(:,:,1)), "lat_resolu=",MAXVAL(resol_lu(:,:,2)), iml, jml
	!
	res_lon=lonrel(2,1)-lonrel(1,1)
	res_lat=latrel(1,1)-latrel(1,2)
	lon0=MINVAL(lonrel)-res_lon/2.0
	lat0=MAXVAL(latrel)+res_lat/2.0
	WRITE(numout,*) 'resolution', res_lon, res_lat
	WRITE(numout,*) 'Coordinate of WEST-NORTH cornor (row1, col1)', lon0, lat0
	gridarea(:)=zero
	std_totsed(:)=zero
	effgrid_perc(:)=zero
	DO ig =1,nbpt
		ip=floor((lalo(ig,2)-lon0)/res_lon)+1
		jp=floor((lat0-lalo(ig,1))/res_lat)+1
		IF ((ip.GT.zero).AND.(ip.LE.iml).AND.(jp.GT.zero).AND.(jp.LE.jml)) THEN
			gridarea(ig)=resol_lu(ip,jp,1)*resol_lu(ip,jp,2)
			std_totsed(ig)=MAX(totsed(ip,jp),0.0)
			effgrid_perc(ig)=MAX(effgrid(ip,jp),0.0)
		ENDIF !((ip.GT.zero).AND.(ip.LE.iml).AND.(jp.GT.zero).AND.(jp.LE.jml)) THEN
		WRITE(numout,*) 'Erosion map reading', ip,jp,gridarea(ig),std_totsed(ig),effgrid_perc(ig)
	ENDDO !DO ig =1,nbpt
	is_root_prc=is_root_prc_old
	!   
!   WRITE(numout,*) 'Effective area', gridarea(:) 
!	WRITE(numout,*) 'Effective grid percent', effgrid_perc(:) 
!   WRITE(numout,*) 'standrad sediment loss rate(ton)', std_totsed(:)
	!
	!	 Variable that will be used in erosion_main(**) should not be deallocated   zhc 
	IF (ALLOCATED(latrel))            		DEALLOCATE (latrel)
	IF (ALLOCATED(lonrel))            		DEALLOCATE (lonrel)
	IF (ALLOCATED(mask))            		DEALLOCATE (mask)
    IF (ALLOCATED(resol_lu))            	DEALLOCATE (resol_lu)
	IF (ALLOCATED(totsed))            		DEALLOCATE (totsed)
	IF (ALLOCATED(ero_index))            	DEALLOCATE (ero_index)
	IF (ALLOCATED(ero_area))            	DEALLOCATE (ero_area)
	IF (ALLOCATED(effgrid))            		DEALLOCATE (effgrid)
    !
  END SUBROUTINE erosionmap 
!
!
!  
END MODULE erosion																																													
																							
